@@ -12,9 +12,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef __GDK_DEVICE_MANAGER_WIN32_H__
@@ -37,8 +35,12 @@ typedef struct _GdkDeviceManagerWin32Class GdkDeviceManagerWin32Class;
 struct _GdkDeviceManagerWin32
 {
   GdkDeviceManager parent_object;
+  /* Master Devices */
   GdkDevice *core_pointer;
   GdkDevice *core_keyboard;
+  /* Fake slave devices */
+  GdkDevice *system_pointer;
+  GdkDevice *system_keyboard;
   GList *wintab_devices;
 };
 
