@@ -44,70 +44,13 @@
 
 /**
  * SECTION:gtkimagemenuitem
- * @Short_description: A deprecated widget for a menu item with an icon
+ * @Short_description: A menu item with an icon
  * @Title: GtkImageMenuItem
  *
  * A GtkImageMenuItem is a menu item which has an icon next to the text label.
  *
- * This is functionally equivalent to:
- *
- * |[<!-- language="C" -->
- *   GtkWidget *box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
- *   GtkWidget *icon = gtk_image_new_from_icon_name ("folder-music-symbolic", GTK_ICON_SIZE_MENU);
- *   GtkWidget *label = gtk_label_new ("Music");
- *   GtkWidget *menu_item = gtk_menu_item_new ();
- *
- *   gtk_container_add (GTK_CONTAINER (box), icon);
- *   gtk_container_add (GTK_CONTAINER (box), label);
- *
- *   gtk_container_add (GTK_CONTAINER (menu_item), box);
- *
- *   gtk_widget_show_all (menu_item);
- * ]|
- *
- * Note that the user may disable display of menu icons using
- * the #GtkSettings:gtk-menu-images setting, so make sure to still
- * fill in the text label. If you want to ensure that your menu items
- * show an icon you are strongly encouraged to use a #GtkMenuItem
- * with a #GtkImage instead.
- *
- * #GtkImageMenuItem has been deprecated since GTK+ 3.10. If you want to
- * display an icon in a menu item, you should use #GtkMenuItem and pack a
- * #GtkBox with a #GtkImage and a #GtkLabel instead. You should also consider
- * using #GtkBuilder and the XML #GMenu description for creating menus, by
- * following the [GMenu guide][https://developer.gnome.org/GMenu/]. You should
- * consider using icons in menu items only sparingly, and for "objects" (or
- * "nouns") elements only, like bookmarks, files, and links; "actions" (or
- * "verbs") should not have icons.
- *
- * Furthermore, if you would like to display keyboard accelerator, you must
- * pack the accel label into the box using gtk_box_pack_end() and align the
- * label, otherwise the accelerator will not display correctly. The following
- * code snippet adds a keyboard accelerator to the menu item, with a key
- * binding of Ctrl+M:
- *
- * |[<!-- language="C" -->
- *   GtkWidget *box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
- *   GtkWidget *icon = gtk_image_new_from_icon_name ("folder-music-symbolic", GTK_ICON_SIZE_MENU);
- *   GtkWidget *label = gtk_accel_label_new ("Music");
- *   GtkWidget *menu_item = gtk_menu_item_new ();
- *   GtkAccelGroup *accel_group = gtk_accel_group_new ();
- *
- *   gtk_container_add (GTK_CONTAINER (box), icon);
- *
- *   gtk_label_set_use_underline (GTK_LABEL (label), TRUE);
- *   gtk_label_set_xalign (GTK_LABEL (label), 0.0);
- *
- *   gtk_widget_add_accelerator (menu_item, "activate", accel_group,
- *                               GDK_KEY_m, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
- *   gtk_accel_label_set_accel_widget (GTK_ACCEL_LABEL (label), menu_item);
- *
- *   gtk_box_pack_end (GTK_BOX (box), label, TRUE, TRUE, 0);
- *
- *   gtk_container_add (GTK_CONTAINER (menu_item), box);
- *
- *   gtk_widget_show_all (menu_item);
- * ]|
+ * Note that the user can disable display of menu icons, so make sure to still
+ * fill in the text label.
  */
 
 
@@ -166,8 +109,6 @@ static void gtk_image_menu_item_get_property         (GObject         *object,
                                                       guint            prop_id,
                                                       GValue          *value,
                                                       GParamSpec      *pspec);
-static void gtk_image_menu_item_screen_changed       (GtkWidget       *widget,
-                                                      GdkScreen       *previous_screen);
 
 static void gtk_image_menu_item_recalculate          (GtkImageMenuItem *image_menu_item);
 
@@ -194,7 +135,6 @@ gtk_image_menu_item_class_init (GtkImageMenuItemClass *klass)
   GtkContainerClass *container_class = (GtkContainerClass*) klass;
 
   widget_class->destroy = gtk_image_menu_item_destroy;
-  widget_class->screen_changed = gtk_image_menu_item_screen_changed;
   widget_class->get_preferred_width = gtk_image_menu_item_get_preferred_width;
   widget_class->get_preferred_height = gtk_image_menu_item_get_preferred_height;
   widget_class->get_preferred_height_for_width = gtk_image_menu_item_get_preferred_height_for_width;
@@ -217,8 +157,7 @@ gtk_image_menu_item_class_init (GtkImageMenuItemClass *klass)
    *
    * Child widget to appear next to the menu text.
    *
-   * Deprecated: 3.10: Use a #GtkMenuItem containing a #GtkBox with
-   *   a #GtkAccelLabel and a #GtkImage instead
+   * Deprecated: 3.10
    */
   g_object_class_install_property (gobject_class,
                                    PROP_IMAGE,
@@ -235,7 +174,7 @@ gtk_image_menu_item_class_init (GtkImageMenuItemClass *klass)
    *
    * Since: 2.16
    *
-   * Deprecated: 3.10: Use a named icon from the #GtkIconTheme instead
+   * Deprecated: 3.10
    */
   g_object_class_install_property (gobject_class,
                                    PROP_USE_STOCK,
@@ -255,8 +194,7 @@ gtk_image_menu_item_class_init (GtkImageMenuItemClass *klass)
    *
    * Since: 2.16
    *
-   * Deprecated: 3.10: Use a #GtkMenuItem containing a #GtkBox with
-   *   a #GtkAccelLabel and a #GtkImage instead
+   * Deprecated: 3.10
    */
   g_object_class_install_property (gobject_class,
                                    PROP_ALWAYS_SHOW_IMAGE,
@@ -273,7 +211,7 @@ gtk_image_menu_item_class_init (GtkImageMenuItemClass *klass)
    *
    * Since: 2.16
    *
-   * Deprecated: 3.10: Use gtk_widget_add_accelerator() instead 
+   * Deprecated: 3.10
    */
   g_object_class_install_property (gobject_class,
                                    PROP_ACCEL_GROUP,
@@ -368,21 +306,6 @@ gtk_image_menu_item_get_property (GObject         *object,
     }
 }
 
-static gboolean
-show_image (GtkImageMenuItem *image_menu_item)
-{
-  GtkImageMenuItemPrivate *priv = image_menu_item->priv;
-  GtkSettings *settings = gtk_widget_get_settings (GTK_WIDGET (image_menu_item));
-  gboolean show;
-
-  if (priv->always_show_image)
-    show = TRUE;
-  else
-    g_object_get (settings, "gtk-menu-images", &show, NULL);
-
-  return show;
-}
-
 static void
 gtk_image_menu_item_map (GtkWidget *widget)
 {
@@ -393,7 +316,7 @@ gtk_image_menu_item_map (GtkWidget *widget)
 
   if (priv->image)
     g_object_set (priv->image,
-                  "visible", show_image (image_menu_item),
+                  "visible", priv->always_show_image,
                   NULL);
 }
 
@@ -731,35 +654,22 @@ gtk_image_menu_item_activatable_interface_init (GtkActivatableIface  *iface)
   iface->sync_action_properties = gtk_image_menu_item_sync_action_properties;
 }
 
-static GtkWidget *
-gtk_image_menu_item_ensure_image (GtkImageMenuItem *item)
-{
-  GtkWidget *image;
-
-  image = gtk_image_menu_item_get_image (item);
-  if (!GTK_IS_IMAGE (image))
-    {
-      image = gtk_image_new ();
-      gtk_widget_show (image);
-      gtk_image_menu_item_set_image (item, image);
-    }
-
-  return image;
-}
-
 static gboolean
 activatable_update_stock_id (GtkImageMenuItem *image_menu_item, GtkAction *action)
 {
+  GtkWidget   *image;
   const gchar *stock_id  = gtk_action_get_stock_id (action);
 
+  image = gtk_image_menu_item_get_image (image_menu_item);
+
   G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
 
-  if (stock_id && gtk_icon_factory_lookup_default (stock_id))
+  if (GTK_IS_IMAGE (image) &&
+      stock_id && gtk_icon_factory_lookup_default (stock_id))
     {
-      GtkWidget *image;
-
-      image = gtk_image_menu_item_ensure_image (image_menu_item);
+      G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
       gtk_image_set_from_stock (GTK_IMAGE (image), stock_id, GTK_ICON_SIZE_MENU);
+      G_GNUC_END_IGNORE_DEPRECATIONS;
       return TRUE;
     }
 
@@ -771,6 +681,7 @@ activatable_update_stock_id (GtkImageMenuItem *image_menu_item, GtkAction *actio
 static gboolean
 activatable_update_gicon (GtkImageMenuItem *image_menu_item, GtkAction *action)
 {
+  GtkWidget   *image;
   GIcon       *icon = gtk_action_get_gicon (action);
   const gchar *stock_id;
   gboolean     ret = FALSE;
@@ -779,11 +690,11 @@ activatable_update_gicon (GtkImageMenuItem *image_menu_item, GtkAction *action)
 
   G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
 
-  if (icon && !(stock_id && gtk_icon_factory_lookup_default (stock_id)))
-    {
-      GtkWidget *image;
+  image = gtk_image_menu_item_get_image (image_menu_item);
 
-      image = gtk_image_menu_item_ensure_image (image_menu_item);
+  if (icon && GTK_IS_IMAGE (image) &&
+      !(stock_id && gtk_icon_factory_lookup_default (stock_id)))
+    {
       gtk_image_set_from_gicon (GTK_IMAGE (image), icon, GTK_ICON_SIZE_MENU);
       ret = TRUE;
     }
@@ -793,21 +704,20 @@ activatable_update_gicon (GtkImageMenuItem *image_menu_item, GtkAction *action)
   return ret;
 }
 
-static gboolean
+static void
 activatable_update_icon_name (GtkImageMenuItem *image_menu_item, GtkAction *action)
 {
+  GtkWidget   *image;
   const gchar *icon_name = gtk_action_get_icon_name (action);
 
-  if (icon_name)
+  image = gtk_image_menu_item_get_image (image_menu_item);
+
+  if (GTK_IS_IMAGE (image) &&
+      (gtk_image_get_storage_type (GTK_IMAGE (image)) == GTK_IMAGE_EMPTY ||
+       gtk_image_get_storage_type (GTK_IMAGE (image)) == GTK_IMAGE_ICON_NAME))
     {
-      GtkWidget *image;
-
-      image = gtk_image_menu_item_ensure_image (image_menu_item);
       gtk_image_set_from_icon_name (GTK_IMAGE (image), icon_name, GTK_ICON_SIZE_MENU);
-      return TRUE;
     }
-
-  return FALSE;
 }
 
 static void
@@ -839,6 +749,7 @@ gtk_image_menu_item_sync_action_properties (GtkActivatable *activatable,
                                             GtkAction      *action)
 {
   GtkImageMenuItem *image_menu_item;
+  GtkWidget *image;
   gboolean   use_appearance;
 
   image_menu_item = GTK_IMAGE_MENU_ITEM (activatable);
@@ -852,6 +763,21 @@ gtk_image_menu_item_sync_action_properties (GtkActivatable *activatable,
   if (!use_appearance)
     return;
 
+  image = gtk_image_menu_item_get_image (image_menu_item);
+  if (image && !GTK_IS_IMAGE (image))
+    {
+      gtk_image_menu_item_set_image (image_menu_item, NULL);
+      image = NULL;
+    }
+
+  if (!image)
+    {
+      image = gtk_image_new ();
+      gtk_widget_show (image);
+      gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (activatable),
+                                     image);
+    }
+
   if (!activatable_update_stock_id (image_menu_item, action) &&
       !activatable_update_gicon (image_menu_item, action))
     activatable_update_icon_name (image_menu_item, action);
@@ -934,7 +860,7 @@ gtk_image_menu_item_new_with_mnemonic (const gchar *label)
  *
  * Returns: a new #GtkImageMenuItem.
  *
- * Deprecated: 3.10: Use gtk_menu_item_new_with_mnemonic() instead.
+ * Deprecated: 3.10: Use gtk_menu_item_new() instead.
  */
 GtkWidget*
 gtk_image_menu_item_new_from_stock (const gchar   *stock_id,
@@ -1006,8 +932,7 @@ gtk_image_menu_item_get_use_stock (GtkImageMenuItem *image_menu_item)
  * @image_menu_item: a #GtkImageMenuItem
  * @always_show: %TRUE if the menuitem should always show the image
  *
- * If %TRUE, the menu item will ignore the #GtkSettings:gtk-menu-images
- * setting and always show the image, if available.
+ * If %TRUE, the menu item will always show the image, if available.
  *
  * Use this property if the menuitem would be useless or hard to use
  * without the image.
@@ -1032,7 +957,7 @@ gtk_image_menu_item_set_always_show_image (GtkImageMenuItem *image_menu_item,
 
       if (priv->image)
         {
-          if (show_image (image_menu_item))
+          if (priv->always_show_image)
             gtk_widget_show (priv->image);
           else
             gtk_widget_hide (priv->image);
@@ -1046,8 +971,7 @@ gtk_image_menu_item_set_always_show_image (GtkImageMenuItem *image_menu_item,
  * gtk_image_menu_item_get_always_show_image:
  * @image_menu_item: a #GtkImageMenuItem
  *
- * Returns whether the menu item will ignore the #GtkSettings:gtk-menu-images
- * setting and always show the image, if available.
+ * Returns whether the menu item will always show the image, if available.
  *
  * Returns: %TRUE if the menu item will always show the image
  *
@@ -1151,10 +1075,9 @@ gtk_image_menu_item_set_image (GtkImageMenuItem *image_menu_item,
 
   gtk_widget_set_parent (image, GTK_WIDGET (image_menu_item));
   g_object_set (image,
-                "visible", show_image (image_menu_item),
+                "visible", priv->always_show_image,
                 "no-show-all", TRUE,
                 NULL);
-  gtk_image_set_pixel_size (GTK_IMAGE (image), 16);
 
   g_object_notify (G_OBJECT (image_menu_item), "image");
 }
@@ -1166,7 +1089,7 @@ gtk_image_menu_item_set_image (GtkImageMenuItem *image_menu_item,
  * Gets the widget that is currently set as the image of @image_menu_item.
  * See gtk_image_menu_item_set_image().
  *
- * Returns: (transfer none): the widget set as image of @image_menu_item
+ * Return value: (transfer none): the widget set as image of @image_menu_item
  *
  * Deprecated: 3.10
  **/
@@ -1205,66 +1128,3 @@ gtk_image_menu_item_remove (GtkContainer *container,
       GTK_CONTAINER_CLASS (gtk_image_menu_item_parent_class)->remove (container, child);
     }
 }
-
-static void
-show_image_change_notify (GtkImageMenuItem *image_menu_item)
-{
-  GtkImageMenuItemPrivate *priv = image_menu_item->priv;
-
-  if (priv->image)
-    {
-      if (show_image (image_menu_item))
-        gtk_widget_show (priv->image);
-      else
-        gtk_widget_hide (priv->image);
-    }
-}
-
-static void
-traverse_container (GtkWidget *widget,
-                    gpointer   data)
-{
-  if (GTK_IS_IMAGE_MENU_ITEM (widget))
-    show_image_change_notify (GTK_IMAGE_MENU_ITEM (widget));
-  else if (GTK_IS_CONTAINER (widget))
-    gtk_container_forall (GTK_CONTAINER (widget), traverse_container, NULL);
-}
-
-static void
-gtk_image_menu_item_setting_changed (GtkSettings *settings)
-{
-  GList *list, *l;
-
-  list = gtk_window_list_toplevels ();
-
-  for (l = list; l; l = l->next)
-    gtk_container_forall (GTK_CONTAINER (l->data),
-                          traverse_container, NULL);
-
-  g_list_free (list);
-}
-
-static void
-gtk_image_menu_item_screen_changed (GtkWidget *widget,
-                                    GdkScreen *previous_screen)
-{
-  GtkSettings *settings;
-  gulong show_image_connection;
-
-  if (!gtk_widget_has_screen (widget))
-    return;
-
-  settings = gtk_widget_get_settings (widget);
-
-  show_image_connection =
-    g_signal_handler_find (settings, G_SIGNAL_MATCH_FUNC, 0, 0,
-                           NULL, gtk_image_menu_item_setting_changed, NULL);
-
-  if (show_image_connection)
-    return;
-
-  g_signal_connect (settings, "notify::gtk-menu-images",
-                    G_CALLBACK (gtk_image_menu_item_setting_changed), NULL);
-
-  show_image_change_notify (GTK_IMAGE_MENU_ITEM (widget));
-}
