@@ -173,7 +173,8 @@ static const GDebugKey gtk_debug_keys[] = {
   {"size-request", GTK_DEBUG_SIZE_REQUEST},
   {"no-css-cache", GTK_DEBUG_NO_CSS_CACHE},
   {"baselines", GTK_DEBUG_BASELINES},
-  {"pixel-cache", GTK_DEBUG_PIXEL_CACHE}
+  {"pixel-cache", GTK_DEBUG_PIXEL_CACHE},
+  {"no-pixel-cache", GTK_DEBUG_NO_PIXEL_CACHE}
 };
 #endif /* G_ENABLE_DEBUG */
 
