@@ -90,6 +90,18 @@ _gtk_pixel_cache_set_extra_size (GtkPixelCache *cache,
   cache->extra_height = extra_height ? extra_height : DEFAULT_EXTRA_SIZE;
 }
 
+void
+_gtk_pixel_cache_get_extra_size (GtkPixelCache *cache,
+                                 guint         *extra_width,
+                                 guint         *extra_height)
+{
+  if (extra_width)
+    *extra_width = cache->extra_width;
+
+  if (extra_height)
+    *extra_height = cache->extra_height;
+}
+
 void
 _gtk_pixel_cache_set_content (GtkPixelCache   *cache,
 			      cairo_content_t  content)
@@ -162,6 +174,11 @@ _gtk_pixel_cache_create_surface_if_needed (GtkPixelCache         *cache,
   cairo_pattern_t *bg;
   double red, green, blue, alpha;
 
+#ifdef G_ENABLE_DEBUG
+  if (gtk_get_debug_flags () & GTK_DEBUG_NO_PIXEL_CACHE)
+    return;
+#endif
+
   content = cache->content;
   if (!content)
     {
